#include <stdlib.h>
#include "DEM128064H.h"		// ovladaci displeje
#include "graphic.h"	//knihovna pro praci v grafickem rezimu
#include "font.h"		//datova struktura


unsigned int NACTI_ADC(unsigned char kanal){ //vraci hodnotu vybraneho kanalu
	ADMUX &= 0xF0; // vlozi cislo kanalu
	ADMUX |= kanal & 0x0F; // vymaskuje nepouzute bity
	ADCSRA |= (1 << ADSC); // start prevodu
	while(ADCSRA & (1 << ADSC));  //ceka na priznak zkonceni prevodu
	return ADC; //navratova hodnota
}


int main(){
	char hodnotaCH1 [5];
	char hodnotaCH2 [5];
	double vypocetCH1 = 0; // vypocet napeti CH1 na vypsani
	double vypocetCH2 = 0; // vypocet napeti CH1 na vypsani
	int neprepsaniCH1 = 5;
	int neprepsaniCH2 = 5;
	double rozsahCH1, rozsahCH2, napetiCH1, napetiCH2, adc0, adc1, adc2, adc3;
	int jednotkyCH1 = 0; // 1=mV, 2=V
	int jednotkyCH2 = 0; // 1=mV, 2=V
	double pametCH1, pametCH2;
	int stisk = 0;
	
	// Nastaveni prevodniku
	ADMUX |= (1 << REFS0) | (1 << REFS1); // 1,1 V vnitrni reference
	ADCSRA |= (1 << ADEN) | (1 << ADIF) | (1 <<ADPS2) | (1 <<ADPS1) | (1 <<ADPS0);  //zapnuti prevodniku, pznak peruen, peddlika 128
	
	// Nastaveni portu pro tlacitko
	DDRB &= ~ (1 << PINB6);
	PORTB |= (1 << PINB6);
	
	DISPLAY_Init(); // inicializace displeje
	
	while(1){
		
				
		pametCH1=0; pametCH2=0; // vynulovani napeti na kanalu CH1 a CH2
		
		// nacteni 50 hodnot do pameti pro lepsi presnost
		for (int i=0; i < 50; i++)
		{
			adc0 = NACTI_ADC(0);
			pametCH1=pametCH1+adc0;
			adc1 = NACTI_ADC(1);
			pametCH2=pametCH2+adc1;
		}
		
		napetiCH1 = (pametCH1/50)*1100/1024; // vypocet napeti na CH1 v mV
		napetiCH2 = (pametCH2/50)*1100/1024; // vypocet napeti na CH2 v mV
		
		//nacteni hodnot prevodniku
		adc2 = (NACTI_ADC(2));
		adc3 = (NACTI_ADC(3));
		rozsahCH1 = adc2*1100/1024; //vypocet napeti na rozsahu CH1
		rozsahCH2 = adc3*1100/1024; //vypocet napeti na rozsahu CH2
		
		// detekuje stisk tlacitka
		if((LCD_CTRL_PIN & (1 << PINB6)) == 0)
		{
			stisk++;
			if (stisk>2){stisk = 0;}
		}
		
//================================= CH1 ========================================================================================================	

// ========== Rozsah 200mV CH1 ===========
		
		if ((rozsahCH1<=1125) && (rozsahCH1>=875)) // napeti na delici pro 200 mV rozsah
		{			
			if (stisk == 0) // jednotky mV
			{
				if (jednotkyCH1 == 2){neprepsaniCH1 = 5;} // z V umozni prepsani na mV
				
				if (stisk != neprepsaniCH1) // neprepise tu samou jednotku
				{
					draw_text(18, 40, 42, 15, "mV", font2); // vypsani jednotky
					neprepsaniCH1 = 0; // zvolena jednotka je mV
				}	
				vypocetCH1=napetiCH1; // vypocet napeti
				jednotkyCH1= 1;	// zvolena jednotka mV			
			}
			
			if (stisk == 1) // jednotky dB
			{
				if (stisk != neprepsaniCH1) // neprepise tu samou jednotku
				{
					draw_text(18, 40, 42, 15, "dB", font2); // vypsani jednotky
					neprepsaniCH1=1; // zvolena jednotka je dB
				}
				vypocetCH1=20*log10(napetiCH1/1000); // vypocet dB
			}
			
			if (stisk == 2) // jednotky dBm
			{
				if (stisk != neprepsaniCH1) // neprepise tu samou jednotku
				{
					draw_text(18, 40, 42, 15, "dBm", font2); // vypsani jednotky
					neprepsaniCH1=2; // zvolena jednotka je dBm
				}
				vypocetCH1=10*log10((pow(napetiCH1/1000,2)/0.6)); // vypocet dBm
			}						
			dtostrf(vypocetCH1, 3, 0, hodnotaCH1); // prevod z double do char
		}

// ========== Rozsah 2V CH1 ===========
		
		if ((rozsahCH1<=875) && (rozsahCH1>=625))
		{
			if (stisk == 0) // jednotky V
			{
				if (jednotkyCH1 == 1){neprepsaniCH1 = 5;}
					
				if (stisk != neprepsaniCH1)
				{
					draw_text(18, 40, 42, 15, " V", font2);
					neprepsaniCH1=0;
				}
				vypocetCH1=napetiCH1/100;
				dtostrf(vypocetCH1, 1, 2, hodnotaCH1);
				jednotkyCH1=2;
			}
			
			if (stisk == 1) // jednotky dB
			{
				if (stisk != neprepsaniCH1)
				{
					draw_text(18, 40, 42, 15, "dB", font2);
					neprepsaniCH1 = 1;
				}
				vypocetCH1=20*log10(napetiCH1/100);
				dtostrf(vypocetCH1, 2, 0, hodnotaCH1);
			}
			
			if (stisk == 2) // jednotky dBm
			{
				if (stisk != neprepsaniCH1)
				{
					draw_text(18, 40, 42, 15, "dBm", font2);
					neprepsaniCH1 = 2;
				}
				vypocetCH1=10*log10((pow(napetiCH1/100,2)/0.6));
				dtostrf(vypocetCH1, 2, 0, hodnotaCH1);
			}					
		}


// ========== Rozsah 20V CH1 ===========
		
		if ((rozsahCH1<=625) && (rozsahCH1>=375))
		{		
			if (stisk == 0) // jednotky V
			{
				if (jednotkyCH1 == 1){neprepsaniCH1 = 5;}
					
				if (stisk != neprepsaniCH1)
				{
					draw_text(18, 40, 42, 15, " V", font2);
					neprepsaniCH1=0;
				}
				vypocetCH1=napetiCH1/10;
				dtostrf(vypocetCH1, 2, 1, hodnotaCH1);
				jednotkyCH1=2;
			}
			
			if (stisk == 1) // jednotky dB
			{
				if (stisk != neprepsaniCH1)
				{
					draw_text(18, 40, 42, 15, "dB", font2);
					neprepsaniCH1=1;
				}
				vypocetCH1=20*log10(napetiCH1/10);
				dtostrf(vypocetCH1, 2, 0, hodnotaCH1);
			}
			
			if (stisk == 2) // jednotky dBm
			{
				if (stisk != neprepsaniCH1)
				{
					draw_text(18, 40, 42, 15, "dBm", font2);
					neprepsaniCH1=2;
				}
				vypocetCH1=10*log10((pow(napetiCH1/10,2)/0.6));
				dtostrf(vypocetCH1, 2, 0, hodnotaCH1);
			}		
		}
		

// ========== Rozsah 300V  CH1 ===========
		
		if ((rozsahCH1<=375) && (rozsahCH1>=125))
		{			
			if (stisk == 0) // jednotky V
			{
				if (jednotkyCH1 == 1){neprepsaniCH1 = 5;}
				
				if (stisk != neprepsaniCH1)
				{
					draw_text(18, 40, 42, 15, " V", font2);
					neprepsaniCH1=0;
				}
				vypocetCH1=0.3*napetiCH1;
				jednotkyCH1= 2;
			}
			
			if (stisk == 1) // jednotky dB
			{
				if (stisk != neprepsaniCH1)
				{
					draw_text(18, 40, 42, 15, "dB", font2);
					neprepsaniCH1=1;
				}
				vypocetCH1=20*log10(0.3*napetiCH1);
			}
			
			if (stisk == 2) // jednotky dBm
			{
				if (stisk != neprepsaniCH1)
				{
					draw_text(18, 40, 42, 15, "dBm", font2);
					neprepsaniCH1=2;
				}
				vypocetCH1=10*log10((pow(0.3*napetiCH1,2)/0.6));			
			}		
			dtostrf(vypocetCH1, 3, 0, hodnotaCH1);
		}

		
//================================= CH2 ========================================================================================================

// ========== Rozsah 200mV CH2 ===========
		
		if ((rozsahCH2<=1125) && (rozsahCH2>=875))
		{
			if (stisk == 0) // jednotky mV
			{
				if (jednotkyCH2 == 2){neprepsaniCH2 = 5;}
				
				if (stisk != neprepsaniCH2)
				{
					draw_text(82, 40, 42, 15, "mV", font2);
					neprepsaniCH2 = 0;
				}
				vypocetCH2=napetiCH2;
				jednotkyCH2 = 1;
			}
			
			if (stisk == 1) // jednotky mdB
			{
				if (stisk != neprepsaniCH2)
				{
					draw_text(82, 40, 42, 15, "dB", font2);
					neprepsaniCH2=1;
				}
				vypocetCH2=20*log10(napetiCH2/1000);
			}
			
			if (stisk == 2) // jednotky mdBm
			{
				if (stisk != neprepsaniCH2)
				{
					draw_text(82, 40, 42, 15, "dBm", font2);
					neprepsaniCH2=2;
				}
				vypocetCH2=10*log10((pow(napetiCH2/1000,2)/0.6));			
			}
			dtostrf(vypocetCH2, 3, 0, hodnotaCH2);
		}

// ========== Rozsah 2V CH2 ===========
		
		if ((rozsahCH2<=875) && (rozsahCH2>=625))
		{
			if (stisk == 0) // jednotky V
			{
				if (jednotkyCH2 == 1){neprepsaniCH2 = 5;}
				
				if (stisk != neprepsaniCH2)
				{
					draw_text(82, 40, 42, 15, " V", font2);
					neprepsaniCH2=0;
				}
				vypocetCH2=napetiCH2/100;
				dtostrf(vypocetCH2, 1, 2, hodnotaCH2);
				jednotkyCH2 = 2;
			}
			
			if (stisk == 1) // jednotky dB
			{
				if (stisk != neprepsaniCH2)
				{
					draw_text(82, 40, 42, 15, "dB", font2);
					neprepsaniCH2 = 1;
				}
				vypocetCH2=20*log10(napetiCH2/100);
				dtostrf(vypocetCH2, 2, 0, hodnotaCH2);
			}
			
			if (stisk == 2) // jednotky dBm
			{
				if (stisk != neprepsaniCH2)
				{
					draw_text(82, 40, 42, 15, "dBm", font2);
					neprepsaniCH2 = 2;
				}
				vypocetCH2=10*log10((pow(napetiCH2/100,2)/0.6));
				dtostrf(vypocetCH2, 2, 0, hodnotaCH2);				
			}						
		}


// ========== Rozsah 20V CH2 ===========
		
		if ((rozsahCH2<=625) && (rozsahCH2>=375))
		{	
			if (stisk == 0) // jednotky V
			{
				if (jednotkyCH2 == 1){neprepsaniCH2 = 5;}
				
				if (stisk != neprepsaniCH2)
				{
					draw_text(82, 40, 42, 15, " V", font2);
					neprepsaniCH2=0;
				}
				vypocetCH2=napetiCH2/10;
				dtostrf(vypocetCH2, 2, 1, hodnotaCH2);
				jednotkyCH2 = 2;	
			}
			
			if (stisk == 1) // jednotky dB
			{
				if (stisk != neprepsaniCH2)
				{
					draw_text(82, 40, 42, 15, "dB", font2);
					neprepsaniCH2=1;
				}
				vypocetCH2=20*log10(napetiCH2/10);
				dtostrf(vypocetCH2, 2, 0, hodnotaCH2);
			}
			
			if (stisk == 2) // jednotky dbm
			{
				if (stisk != neprepsaniCH2)
				{
					draw_text(82, 40, 42, 15, "dBm", font2);
					neprepsaniCH2=2;
				}
				vypocetCH2=10*log10((pow(napetiCH2/10,2)/0.6));
				dtostrf(vypocetCH2, 2, 0, hodnotaCH2);	
			}
		}
		

// ========== Rozsah 300V CH2  ===========
		
		if ((rozsahCH2<=375) && (rozsahCH2>=125))
		{			
			if (stisk == 0) // jednotky V
			{
				if (jednotkyCH2 == 1){neprepsaniCH2 = 5;}
				
				if (stisk != neprepsaniCH2)
				{
					draw_text(82, 40, 42, 15, " V", font2);
					neprepsaniCH2=0;
				}
				vypocetCH2=0.3*napetiCH2;
				jednotkyCH2 = 2;	
			}
			
			if (stisk == 1) // jednotky dB
			{
				if (stisk != neprepsaniCH2)
				{
					draw_text(82, 40, 42, 15, "dB", font2);
					neprepsaniCH2=1;
				}
				vypocetCH2=20*log10(0.3*napetiCH2);
			}
			
			if (stisk == 2) // jednotky dbm
			{
				if (stisk != neprepsaniCH2)
				{
					draw_text(82, 40, 42, 15, "dBm", font2);
					neprepsaniCH2=2;
				}
				vypocetCH2=10*log10((pow(0.3*napetiCH2,2)/0.6));	
			}					
			dtostrf(vypocetCH2, 3, 0, hodnotaCH2);	
		}
		
		_delay_ms(40);
	
	
// vypsani hodnot signalu
		
		if ((rozsahCH1<=375) && (rozsahCH1>=125))
		{	
			if ((napetiCH1 > 980) || ((napetiCH1 == 0) && !(stisk == 0)))
			{
				draw_text(8, 1, 50, 26, "- -", font1);
			}
			else
			{
				draw_text(8, 1, 50, 26, hodnotaCH1, font1);
			}
		}
		else
		{
			if ((napetiCH1 > 200) || ((napetiCH1 == 0) && !(stisk == 0)))
			{
				draw_text(8, 1, 50, 26, "- -", font1);
			}
			else
			{
				draw_text(8, 1, 50, 26, hodnotaCH1, font1);
			}
		}
	
		if ((rozsahCH2<=375) && (rozsahCH2>=125))
		{	
			if ((napetiCH2 > 980) || ((napetiCH2 == 0) && !(stisk == 0)))
			{
				draw_text(72, 1, 50, 26, "- -", font1);
			}
			else
			{	
				draw_text(72, 1, 50, 26, hodnotaCH2, font1);
			}			
		}	
		else
		{
			if ((napetiCH2 > 200) || ((napetiCH2 == 0) && !(stisk == 0)))
			{
				draw_text(72, 1, 50, 26, "- -", font1);
			}
			else
			{
				draw_text(72, 1, 50, 26, hodnotaCH2, font1);
			}
		}	
	}
}
